﻿
using System.Collections.Generic;
using System.Text;
using EnvelopeCreatorProject.DocusignAPI;
using EnvelopeCreatorProject.DocusignAPI.Authentication;
using EnvelopeCreatorProject.DocusignAPI.RecipientType;
using EnvelopeCreatorProject.Exceptions;
using EnvelopeCreatorProject.Models;
using EnvelopeCreatorProject.Models.Recipients.IdentityCheck;
using EnvelopeCreatorProject.Models.Signers;
using EnvelopeCreatorProject.Test.Classes;
using NUnit.Framework;
using EnvelopeSettings = EnvelopeCreatorProject.Models.EnvelopeSettings;

namespace EnvelopeCreatorProject.Test
{
    class GenerateEnvelopeTest
    {
        private EnvelopeData _envelopeData;
        private AccountAuthentication _account;
        private EnvelopeTransporterCreator _envelopeTransporterCreator;
        private AccountAuthentication _soboAccount;
        private DocusignTokenHandler _dth;
        private string _apiVersion = "v2";
        private string _integratorKey = "EFFI-7f0f9859-fd10-40cf-9759-ea4e2a320cb3";
        [SetUp]
        public void Initialize()
        {
            _account = new AccountAuthentication
            {
                EnvironmentURL = "demo.docusign.net",
                SenderToken = "",
                AuthToken = "sE4WJCHg4fVhG331CARww0eoiIo=",
                SOBOFlag = false
            };
                
            _soboAccount = new DataGenerator().GetCorrectSoboAccountAuth();
            _envelopeTransporterCreator = new EnvelopeTransporterCreator(_integratorKey, _apiVersion);
            _dth = new DocusignTokenHandler(_integratorKey, _apiVersion);

        }

        [Test]
        public void BuildEnvelopeMail()
        {
            var sBody = "MailBody";
            var sSubject = "MailSubject";
            StringBuilder resultJsonBody = new StringBuilder();
            StringBuilder correctJasonBody = new StringBuilder();

            correctJasonBody = new DataGenerator().GetBuildedMail(correctJasonBody, sBody, sSubject);
            var builder = new Models.Builders.EnvelopeMailBuilder(sBody, sSubject);
            builder.BuildTo(resultJsonBody);
            Assert.AreEqual(correctJasonBody.ToString(), resultJsonBody.ToString());
        }

        [Test]
        public void BuildEnvelopeDocument()
        {
            List<DocusignDocument> lstDoc = new DataGenerator().GetCorrectDocuments();
            StringBuilder resultJsonBody = new StringBuilder();
            StringBuilder correctJasonBody = new StringBuilder();

            correctJasonBody = new DataGenerator().GetBuildedDocument(correctJasonBody, lstDoc);
            var builder = new Models.Builders.EnvelopeDocumentBuilder(lstDoc);
            builder.BuildTo(resultJsonBody);
            Assert.AreEqual(correctJasonBody.ToString(), resultJsonBody.ToString());
        }

        [TestCase("Signer")]
        [TestCase("Agent")]
        [TestCase("CarbonCopiesBuilder")]
        [TestCase("CertifiedDelivery")]
        [TestCase("Editor")]
        [TestCase("InPersonSigner")]
        [TestCase("Intermediary")]
        public void BuildEnvelopeRecipient(string recipientType)
        {
            List<Recipient> lstRec = new DataGenerator().GetCorrectLstRecipients(recipientType);
            List<Tab> tabs = new DataGenerator().GetCorrectTabs();
            List<Tab> dateSignedFields = new DataGenerator().GetCorrectDateSignedFields();
            HostSigner host = new DataGenerator().GetHostSigner();
            StringBuilder resultJsonBody = new StringBuilder();
            StringBuilder correctJasonBody = new StringBuilder();

            correctJasonBody = new DataGenerator().GetBuildedRecipient(correctJasonBody, lstRec, tabs,
                dateSignedFields, host);
            var builder = new Models.Builders.EnvelopeRecipientBuilder(lstRec, tabs,
                dateSignedFields, host);
            builder.BuildTo(resultJsonBody);
            Assert.AreEqual(correctJasonBody.ToString(), resultJsonBody.ToString());
        }

        [Test]
        public void BuildEnvelopeSettingsAllFalse()
        {
            EnvelopeSettings settings = new DataGenerator().GetAllFalseSettings();
            StringBuilder resultJsonBody = new StringBuilder();
            StringBuilder correctJasonBody = new StringBuilder();

            correctJasonBody = new DataGenerator().GetBuildedSettings(correctJasonBody, settings);
            var builder = new Models.Builders.EnvelopeSettingsBuilder(settings);
            builder.BuildTo(resultJsonBody);
            Assert.AreEqual(correctJasonBody.ToString(), resultJsonBody.ToString());
        }

        [Test]
        public void BuildEnvelopeStatus()
        {
            StringBuilder resultJsonBody = new StringBuilder();
            StringBuilder correctJasonBody = new StringBuilder();

            correctJasonBody.AppendLine("\"status\":\"sent\",");
            var builder = new Models.Builders.EnvelopeStatusBuilder();
            builder.BuildTo(resultJsonBody);
            Assert.AreEqual(correctJasonBody.ToString(), resultJsonBody.ToString());
        }

        [Test]
        public void GetAccountInfoWithoutTokenOrIncorrectToken()
        {
            var result = _dth.getAccountInformation("mbuUSPHiLMj8utEAxOYVdyCACd8", "demo.docusign.net");
            Assert.IsTrue(result == null);
        }

        [Test]
        public void GetAccountInfoWithCorrectToken()
        {
            var result = _dth.getAccountInformation("YLdsaGIREo1JPs+Aci3FjOKNcGA=", "demo.docusign.net");
            Assert.IsTrue(result.loginAccounts.Count > 0);
        }

        [Test]
        public void GetDocusignOauth2Token()
        {
            var result = _dth.getDocusignOauth2Token("hpazos@virtualmindsoftware.com", "virtualmind", "demo.docusign.net");
            var resultRevoke = _dth.revokeDocusignOauth2Token(result, "demo.docusign.net");
            Assert.IsTrue(resultRevoke == 0);
        }

        [TestCase("Signer")]
        [TestCase("Agent")]
        [TestCase("CarbonCopiesBuilder")]
        [TestCase("CertifiedDelivery")]
        [TestCase("Editor")]
        [TestCase("InPersonSigner")]
        public void GenerateEnvelopeWithCompleteData(string recipientType)
        {
            _envelopeData = new DataGenerator().GetCorrectData(recipientType);
            if (recipientType == "InPersonSigner")
            {
                _envelopeData.Host.HostEmail = "hpazos@virtualmindsoftware.com";
                _envelopeData.Host.HostName = "Hernan";
            }
            var result = _envelopeTransporterCreator.CreateEnvelope(_account, _envelopeData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }

        [TestCase("Signer")]
        [TestCase("Agent")]
        [TestCase("CarbonCopiesBuilder")]
        [TestCase("CertifiedDelivery")]
        [TestCase("Editor")]
        public void GenerateEnvelopeWithCompleteDataSoboSigner(string recipientType)
        {
            _envelopeData = new DataGenerator().GetCorrectData(recipientType);
            var result = _envelopeTransporterCreator.CreateEnvelope(_soboAccount, _envelopeData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }

        [TestCase(true)]
        [TestCase(false)]
        public void GenerateEnvelopeWithNoAuthTokenSoboSigner(bool sobo)
        {
            var result = _envelopeTransporterCreator.CreateEnvelope(new AccountAuthentication() { SOBOFlag = sobo }, null);
            Assert.AreEqual(ErrorCode.OAuthError, result.ErrorCode);
        }

        [Test]
        public void RecipientWithNoType()
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                Host = generator.GetHostSignerNull(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "a@a.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1
                    }
                }
            };
            var result = _envelopeTransporterCreator.CreateEnvelope(_account, eData);
            Assert.AreEqual(ErrorCode.InvalidRecipientType, result.ErrorCode);
        }

        [TestCase(true, null, null)]
        [TestCase(false, "hostName", null)]
        [TestCase(false, null, "hostEmail")]
        [TestCase(false, null, null)]
        public void InPersonSignerRecipientWithNoHostInformation(bool nullHost, string hostName, string hostEmail)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                Host = nullHost ? null : new HostSigner() { HostName = hostName, HostEmail = hostEmail },
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "a@a.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new InPersonSigner()
                    }
                }
            };
            var result = _envelopeTransporterCreator.CreateEnvelope(_account, eData);
            Assert.AreEqual(ErrorCode.InvalidSignerHost, result.ErrorCode);
        }

        [TestCase("hostName", "hostEmail")]
        public void InPersonSignerRecipientWithBadNoHostInformation(string hostName, string hostEmail)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                Host = new HostSigner() { HostName = hostName, HostEmail = hostEmail },
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "a@a.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new InPersonSigner()
                    }
                }
            };
            var result = _envelopeTransporterCreator.CreateEnvelope(_account, eData);
            Assert.AreEqual(ErrorCode.DocusignError, result.ErrorCode);
        }

        [TestCase("1111")]
        public void RecipientWithBadPhoneAuthenticationFails(string phone)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "docusigntests@mailinator.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new Signer(),
                        RecipientAuthentication = new PhoneRecipientAuthentication()
                        {
                            IDCheckConfigurationName  = "Phone Auth $",
                            SenderProvidedNumbers = new List<string>(){phone},
                            RequireIDLookup = true
                        }
                    }
                }
            };
            var result = _envelopeTransporterCreator.CreateEnvelope(_account, eData);
            Assert.AreEqual(ErrorCode.DocusignError, result.ErrorCode);
        }
        [TestCase("2025555487")]
        public void RecipientWithPhoneAuthentication(string phone)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "docusigntests@mailinator.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new Signer(),
                        RecipientAuthentication = new PhoneRecipientAuthentication()
                        {
                            IDCheckConfigurationName  = "Phone Auth $",
                            SenderProvidedNumbers = new List<string>(){phone},
                            RequireIDLookup = true
                        }
                    }
                }
            };
            var account = new AccountAuthentication
            {
                EnvironmentURL = "demo.docusign.net",
                SenderToken = "",
                AuthToken = "48iYjQ6wOn3MgJ938M0ORqBkX2M=",
                SOBOFlag = false
            };

            var result = _envelopeTransporterCreator.CreateEnvelope(account, eData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }
        [TestCase("2025555487")]
        public void RecipientWithSMSAuthentication(string phone)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "docusigntests@mailinator.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new Signer(),
                        RecipientAuthentication = new SmsRecipientAuthentication()
                        {
                            IDCheckConfigurationName  = "SMS Auth $",
                            SenderProvidedNumbers = new List<string>(){phone},
                            RequireIDLookup = true
                        }
                    }
                }
            };
            var account = new AccountAuthentication
            {
                EnvironmentURL = "demo.docusign.net",
                SenderToken = "",
                AuthToken = "48iYjQ6wOn3MgJ938M0ORqBkX2M=",
                SOBOFlag = false
            };

            var result = _envelopeTransporterCreator.CreateEnvelope(account, eData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }

        [TestCase("ID Check $")]
        [TestCase("Live ID")]
        [TestCase("Facebook")]
        [TestCase("Linkedin")]
        [TestCase("Google")]
        [TestCase("Salesforce")]
        [TestCase("Twitter")]
        [TestCase("Yahoo")]
        [TestCase("OpenID")]
        public void RecipientWithAuthentication(string auth)
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = generator.GetCorrectTabs(),
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "docusigntests@mailinator.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new Signer(),
                        RecipientAuthentication = new RecipientAuthentication()
                        {
                            IDCheckConfigurationName  = auth,
                            RequireIDLookup = true
                        }
                    }
                }
            };
            var account = new AccountAuthentication
            {
                EnvironmentURL = "demo.docusign.net",
                SenderToken = "",
                AuthToken = "48iYjQ6wOn3MgJ938M0ORqBkX2M=",
                SOBOFlag = false
            };

            var result = _envelopeTransporterCreator.CreateEnvelope(account, eData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }

        [Test]
        public void OptionalSignatures()
        {
            var generator = new DataGenerator();
            var eData = new EnvelopeData
            {
                MailSubject = "Sign Subject Test",
                MailBody = "Sign Body for Test",
                Documents = generator.GetCorrectDocuments(),
                Tabs = new List<Tab>()
                {
                    new Tab
                    {
                        RecipientID = "1",
                        PageNumber = "5",
                        XPosition = "41",
                        YPosition = "341",
                        Type = TabTypeCode.SignHere,
                        DocumentID = "1",
                        Optional = true
                    }
                },
                DateSignedFields = generator.GetCorrectDateSignedFields(),
                EnvelopeSettings = generator.GetAllFalseSettings(),
                Recipients = new List<Recipient>()
                {
                    new Recipient()
                    {
                        RoleName = "1own",
                        Email = "docusigntests@mailinator.com",
                        ID = "1",
                        UserName = "tebi",
                        RoutingOrder = 1,
                        RecipientType = new Signer()
                    }
                }
            };
            var account = new AccountAuthentication
            {
                EnvironmentURL = "demo.docusign.net",
                SenderToken = "",
                AuthToken = "48iYjQ6wOn3MgJ938M0ORqBkX2M=",
                SOBOFlag = false
            };

            var result = _envelopeTransporterCreator.CreateEnvelope(account, eData);
            Assert.AreEqual(ErrorCode.Success, result.ErrorCode);
        }
        
    }
}
